/*
xpopclk Version 1.1.0 - World Population Clock
for X Windows.
Copyright (C) 2001-2010  dondalah721@yahoo.com (Dondalah)

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

/* to define the escape key */
#define XK_MISCELLANY 1
#define XK_LATIN1 1

#include <stdio.h>
#include <ctype.h>
#include <string.h>
#include <time.h>
#include <errno.h>
#include <sys/times.h>
#include <X11/Xlib.h>
#include <X11/keysymdef.h>
#include <assert.h>
#include <unistd.h>
#include <stdlib.h>
#include <math.h>

/* Slackware font */
/* #define FONT "linux8x16" */

/* Debian font */
/* #define FONT "8x16" */

/* Debian font */
#define CFONT "6x12"

#define NIL (0)
#define SCRW 240
#define SCRH 160
#define NOTIME 0
#define RELATIVE 1
#define ASCII 2
#define JULIAN 3

unsigned int dayticks;
double totpop;
double dblanndlta,dbltkdlta;
double dbltps,dbltpy;
time_t origclk,frstsecs,totsecs;

int whiteColor,blackColor;
int runflg,resetsw,timesw,prevtime;
int helpsw,returnsw,aboutsw;
int exposesw;
int titlesw,ratesw;
Display *dpy;
Window w;
GC gc;
Font fontid;
Font cfontid;
int cap_loc;
char caption[128];

void putstx(pgm)
char *pgm;
   {
   fprintf(stderr,"Usage: %s <annual_change>\n",
      pgm);
   fprintf(stderr,"Where annual_change is the "
      "annual increase or decrease\n");
   fprintf(stderr,"from -999999999999999 to "
      "999999999999999\n");
   exit(1);
   } /* putstx */

void slp()
   {
   int rslt;
   struct timespec request;
   struct timespec remain;
   request.tv_sec  = (long int) 0;
   request.tv_nsec = (long int) 1;
   rslt = nanosleep(&request,&remain);
   if (rslt < 0)
      {
      fprintf(stderr,"Nanosleep error %d\n",
         errno);
      perror("Nanosleep error");
      exit(1);
      } /* nanosleep error */
   } /* slp using nanosleep */

/* X Windows code is based on: */
/* http://tronche.lri.fr:8000/gui/x/xlib-tutorial/2nd-program-anatomy.html */

void getkey()
   {
   int msk;
   int symbol;
   int XCheckMaskEvent();
   XEvent e;
   XKeyEvent *k;

   msk = KeyPressMask|ExposureMask;

   XSelectInput(dpy, w, msk);

   while (XCheckMaskEvent(dpy, msk, &e))
      {
      if (e.type == KeyPress)
         {
         k = (XKeyEvent *) &e;
         symbol = XLookupKeysym(k,0);
         resetsw = 0;
         if (symbol == XK_Escape
	    || symbol == XK_q)
            {
            runflg = 0;
            } /* if quit */
         else if (symbol == XK_t)
            {
            prevtime = timesw++;
            if (timesw > JULIAN)
               timesw = NOTIME;
            } /* if toggle time */
         else if (symbol == XK_r)
            { /* if reset counts */
            resetsw = 1;
            } /* if reset counts */
         else if (symbol == XK_Return)
            { /* if return */
            returnsw = 1;
            } /* if return */
         else if (symbol == XK_h
            || symbol == ' ')
            { /* if help */
            helpsw ^= 1;
            } /* if help */
         else if (symbol == XK_a)
            { /* if about */
            aboutsw ^= 1;
            } /* if about */
         else if (symbol == XK_n)
            { /* if annual rate */
            ratesw = 1;
            } /* if annual rate */
         else if (symbol == XK_c)
            { /* if caption */
            titlesw = 1;
            } /* if caption */
         } /* if keypress event */
      else if (e.type == Expose)
         {
	 /* 30 40 */
         XSetBackground(dpy, gc, whiteColor);
         XSetForeground(dpy, gc, blackColor);
         XDrawImageString(dpy,w,gc,cap_loc,40,
            caption, strlen(caption));
         } /* if expose event */
      } /* if event received */
   } /* getkey */

int readkb()
   {
   int msk;
   int symbol;
   XEvent e;
   XKeyEvent *k;

   msk = KeyPressMask|ExposureMask;

   XSelectInput(dpy, w, msk);

   while (1)
      {
      XNextEvent(dpy, &e);
      if (e.type == KeyPress)
         {
         k = (XKeyEvent *) &e;
	 symbol = XLookupKeysym(k,0);
	 if (symbol == XK_Return)
	    return(10);
	 else if (symbol == XK_Escape)
	    return(0x1b);
	 else if (symbol == XK_BackSpace)
	    return(8);
	 else if (symbol < ' ' || symbol > '~')
	    continue;
	 else return(symbol);
         } /* if keypress event */
      else if (e.type == Expose)
         {
	 exposesw = 1;
	 return(-1);
         } /* if expose event */
      } /* if event received */
   } /* readkb */

void initx()
   {
   char title[64];

   dpy = XOpenDisplay(NIL);

   if (dpy == NULL)
      {
      fprintf(stderr,"X Windows failure\n");
      exit(1);
      } /* if X Windows is not active */

   assert(dpy);

   whiteColor = WhitePixel(dpy, DefaultScreen(dpy));
   blackColor = BlackPixel(dpy, DefaultScreen(dpy));

   w = XCreateSimpleWindow(dpy,
      DefaultRootWindow(dpy),
      0, 0, 
      SCRW, SCRH,
      0, whiteColor,
      whiteColor);

   XSelectInput(dpy, w, StructureNotifyMask);

   XMapWindow(dpy, w);

   gc = XCreateGC(dpy, w, 0, NIL);

   cfontid = (Font) XLoadFont(dpy,CFONT);

   fontid = (Font) XLoadFont(dpy,FONT);

   XSetFont(dpy,gc,fontid);

   XSetForeground(dpy, gc, blackColor);

   XSetWindowBorderWidth(dpy, w, 40);

   sprintf(title,"xpopclk");
   XStoreName(dpy,w,title);
   XSetIconName(dpy,w,title);

#ifdef UPPERLEFT
   XMoveWindow(dpy,w,0,0);
#endif

   while(1)
      {
      XEvent e;
      XNextEvent(dpy, &e);
      if (e.type == MapNotify) break;
      } /* wait for window initialization */
   } /* initx */

void chg_title()
   {
   int ch;
   char *p,*q;
   char str[128];
   char blnk[128];
   char heading[128];
   XSetBackground(dpy, gc, whiteColor);
   XSetForeground(dpy, gc, whiteColor);
   XFillRectangle(dpy,w,gc,0,0,SCRW,SCRH);
   XSetForeground(dpy, gc, blackColor);
   p = blnk;
   q = blnk + 126;
   while (p < q) *p++ = ' ';
   *p     = '\0';
   p = str;
   q = str + 30;
   *p     = '_';
   *(p+1) = '\0';
   strcpy(heading,"Enter Caption:");
   XDrawImageString(dpy,w,gc,20,30,heading,strlen(heading));
   XDrawImageString(dpy,w,gc,20,50,blnk,strlen(str)+4);
   XDrawImageString(dpy,w,gc,20,50,str,strlen(str));
   ch = 0;
   while (p < q && ch != '\n' && ch != 0x1b)
      {
      XDrawImageString(dpy,w,gc,20,50,blnk,strlen(str)+4);
      XDrawImageString(dpy,w,gc,20,50,str,strlen(str));
      ch = readkb();
      if (ch == 0x1b)
	 {
	 p = str;
	 *p = '\0';
	 break;
	 } /* if cancel */
      if (ch == '\n') break;
      if (ch == 0x08)
	 {
	 if (p > str)
	    {
   	    p--;
	    *p = '_';
	    *(p+1) = ' ';
	    *(p+2) = ' ';
	    *(p+3) = '\0';
	    } /* if not at beginning */
	 continue;
	 } /* if back space */
      if (ch >= ' ' && ch <= '~')
	 {
         *p     = ch;
         *(p+1) = '_';
         *(p+2) = '\0';
         p++;
	 } /* if valid ASCII */
      else return;
      } /* for each char */

   *p = '\0';  /* override cursor */
   resetsw = titlesw = ratesw = returnsw = helpsw = 0;
   XSetForeground(dpy, gc, whiteColor);
   XFillRectangle(dpy,w,gc,0,0,SCRW,SCRH);
   XSetForeground(dpy, gc, blackColor);
   if (!strlen(str) || p <= str)
      {
      XDrawImageString(dpy,w,gc,cap_loc,40,
         caption, strlen(caption));
      return;
      } /* if no change */
   strcpy(caption,str);
   cap_loc = (SCRW >> 1) -
      (((strlen(caption) >> 1) + 1) * 8);
   XDrawImageString(dpy,w,gc,cap_loc,40,
      caption, strlen(caption));
   } /* chg_title */

void chg_rate()
   {
   int ch;
   struct tms t;
   double sign;
   double prevdlta;
   char *p,*q;
   char str[128];
   char blnk[128];
   char heading[128];
   char msg[128];
   XSetBackground(dpy, gc, whiteColor);
   XSetForeground(dpy, gc, whiteColor);
   XFillRectangle(dpy,w,gc,0,0,SCRW,SCRH);
   XSetForeground(dpy, gc, blackColor);
   p = blnk;
   q = blnk + 126;
   while (p < q) *p++ = ' ';
   *p  = '\0';
   p   = str;
   q   = str + 20;
   prevdlta = dblanndlta;
   sign = 1;
   dblanndlta = 0.0;
   *p     = '_';
   *(p+1) = '\0';
   strcpy(heading,"Enter Annual Rate:");
   XDrawImageString(dpy,w,gc,20,30,heading,strlen(heading));
   XDrawImageString(dpy,w,gc,20,50,blnk,strlen(str)+4);
   XDrawImageString(dpy,w,gc,20,50,str,strlen(str));
   ch = 0;
   while (p < q && ch != '\n' && ch != 0x1b)
      {
      XDrawImageString(dpy,w,gc,20,50,blnk,strlen(str)+4);
      XDrawImageString(dpy,w,gc,20,50,str,strlen(str));
      ch = readkb();
      if (ch == 0x1b)
	 {
	 dblanndlta = prevdlta;
	 break;
	 } /* if cancel */
      if (ch == '\n') break;
      if (ch == 0x08)
	 {
	 if (p > str)
	    {
   	    p--;
	    *p = '_';
	    *(p+1) = ' ';
	    *(p+2) = ' ';
	    *(p+3) = '\0';
	    dblanndlta = floor(dblanndlta / 10.0);
	    } /* if not at beginning */
	 else sign = 1;
	 continue;
	 } /* if back space */
      if (ch == '-' && p <= str)
	 {
	 sign   = -1;
	 } /* if sign */
      else if (isdigit(ch))
	 dblanndlta = (dblanndlta * 10.0)
	 + (double) ch - '0';
      else
	 {
	 dblanndlta = prevdlta;
	 return;
	 } /* if invalid entry */
      *p     = ch;
      *(p+1) = '_';
      *(p+2) = '\0';
      p++;
      if (dblanndlta >= 1.0e15) return;
      } /* for each digit */

   if (p <= str || dblanndlta == 0.0)
      {
      dblanndlta = prevdlta;
      } /* if no input */

   if (dblanndlta < 1.0e7)
      {
      sprintf(msg,"Warning low annual rate");
      XDrawImageString(dpy,w,gc,20,70,msg,strlen(msg));
      sprintf(msg,"%.0f", dblanndlta);
      XDrawImageString(dpy,w,gc,20,90,msg,strlen(msg));
      sprintf(msg,"Re-enter annual rate?");
      XDrawImageString(dpy,w,gc,20,110,msg,strlen(msg));
      sprintf(msg,"Press Y or N");
      XDrawImageString(dpy,w,gc,20,130,msg,strlen(msg));
      ch = readkb();
      if (ch == 'y' || ch == 'Y')
	 {
	 chg_rate();
	 return;
	 } /* if recursion */
      } /* warning */
   if (dblanndlta != prevdlta)
      {
      dblanndlta *= sign;
      dbltkdlta = dblanndlta / dbltpy;
      origclk = times(&t);
      time(&frstsecs);
      totsecs = 0;
      totpop = 0.0;
      } /* if rate changed */

   resetsw = ratesw = returnsw = helpsw = 0;
   XSetBackground(dpy, gc, whiteColor);
   XSetForeground(dpy, gc, whiteColor);
   XFillRectangle(dpy,w,gc,0,0,SCRW,SCRH);
   XSetForeground(dpy, gc, blackColor);
   XDrawImageString(dpy,w,gc,cap_loc,40,
      caption, strlen(caption));
   } /* chg_rate */

void helpmenu()
   {
   int prevtimesw;
   int prevaboutsw;
   int prevratesw;
   int prevtitlesw;
   char str[128];
   ratesw = titlesw = 0;
   prevtimesw  = timesw;
   prevaboutsw = aboutsw;
   prevratesw  = ratesw;
   prevtitlesw = titlesw;
   XSetBackground(dpy, gc, whiteColor);
   XSetForeground(dpy, gc, whiteColor);
   XFillRectangle(dpy,w,gc,0,0,SCRW,SCRH);
   XSetForeground(dpy, gc, blackColor);
   while (runflg && !returnsw && helpsw
      && timesw  == prevtimesw
      && aboutsw == prevaboutsw
      && ratesw  == prevratesw
      && titlesw == prevtitlesw
      && !resetsw)
      {
      strcpy(str,"t) Change time mode");
      XDrawImageString(dpy,w,gc,20,30,str,strlen(str));
      strcpy(str,"r) Reset population count");
      XDrawImageString(dpy,w,gc,20,50,str,strlen(str));
      strcpy(str,"a) About xpopclk");
      XDrawImageString(dpy,w,gc,20,70,str,strlen(str));
      strcpy(str,"n) Change annual rate");
      XDrawImageString(dpy,w,gc,20,90,str,strlen(str));
      strcpy(str,"c) Change title");
      XDrawImageString(dpy,w,gc,20,110,str,strlen(str));
      strcpy(str,"Return) Return");
      XDrawImageString(dpy,w,gc,20,130,str,strlen(str));
      while (runflg && !returnsw && helpsw
         && timesw == prevtimesw
         && aboutsw == prevaboutsw
         && ratesw == prevratesw
         && titlesw == prevtitlesw
         && !resetsw)
         {
         getkey();
         slp();
         } /* wait loop */
      } /* help loop */
   if (ratesw) chg_rate();
   if (titlesw) chg_title();
   titlesw = ratesw = returnsw = helpsw = 0;
   XSetBackground(dpy, gc, whiteColor);
   XSetForeground(dpy, gc, whiteColor);
   XFillRectangle(dpy,w,gc,0,0,SCRW,SCRH);
   XSetForeground(dpy, gc, blackColor);
   XDrawImageString(dpy,w,gc,cap_loc,40,
      caption, strlen(caption));
   } /* helpmenu */

void disclaim()
   {
   int prevaboutsw;
   int prevhelpsw;
   char str[128];
   XSetFont(dpy,gc,cfontid);
   XSetBackground(dpy, gc, whiteColor);
   XSetForeground(dpy, gc, whiteColor);
   XFillRectangle(dpy,w,gc,0,0,SCRW,SCRH);
   XSetForeground(dpy, gc, blackColor);
   prevaboutsw = aboutsw;
   prevhelpsw = helpsw;
   while (runflg && !returnsw
      && aboutsw == prevaboutsw
      && helpsw == prevhelpsw)
      {
      strcpy(str,"xpopclk.c version 1.1.0");
      XDrawImageString(dpy,w,gc,5,15,str,strlen(str));
      strcpy(str,"Copyright (C) 2001-2010 ");
      XDrawImageString(dpy,w,gc,5,30,str,strlen(str));
      strcpy(str,"dondalah721@yahoo.com (Dondalah)");
      XDrawImageString(dpy,w,gc,5,45,str,strlen(str));
      strcpy(str,"popclk.c comes with");
      XDrawImageString(dpy,w,gc,5,60,str,strlen(str));
      strcpy(str,"ABSOLUTELY NO WARRANTY");
      XDrawImageString(dpy,w,gc,5,75,str,strlen(str));
      strcpy(str,"For details, read the GNU ");
      XDrawImageString(dpy,w,gc,5,90,str,strlen(str));
      strcpy(str,"Public License at");
      XDrawImageString(dpy,w,gc,5,105,str,strlen(str));
      strcpy(str,"http://www.gnu.org");
      XDrawImageString(dpy,w,gc,5,120,str,strlen(str));
      strcpy(str,"   /copyleft/gpl.html");
      XDrawImageString(dpy,w,gc,5,135,str,strlen(str));
      strcpy(str,"Press return to continue.");
      XDrawImageString(dpy,w,gc,5,150,str,strlen(str));
      while (runflg && !returnsw
         && aboutsw == prevaboutsw
         && helpsw == prevhelpsw)
         {
         getkey();
         slp();
         } /* wait loop */
      } /* disclaim loop */
   returnsw = helpsw = aboutsw = 0;
   XSetBackground(dpy, gc, whiteColor);
   XSetForeground(dpy, gc, whiteColor);
   XFillRectangle(dpy,w,gc,0,0,SCRW,SCRH);
   XSetForeground(dpy, gc, blackColor);
   XSetFont(dpy,gc,fontid);
   XDrawImageString(dpy,w,gc,cap_loc,40,
      caption, strlen(caption));
   } /* disclaim */

int gettps()
   {
   unsigned int diff;
   time_t frstsec,currsec;
   clock_t prev,clk;
   struct tms t;
   char str[128];
   XSetFont(dpy,gc,cfontid);
   XSetBackground(dpy, gc, whiteColor);
   XSetForeground(dpy, gc, blackColor);
   strcpy(str,"h = Help");
   XDrawImageString(dpy,w,gc,20,20,str,strlen(str));
   strcpy(str,"t = Time mode");
   XDrawImageString(dpy,w,gc,20,35,str,strlen(str));
   strcpy(str,"r = Reset count");
   XDrawImageString(dpy,w,gc,20,50,str,strlen(str));
   strcpy(str,"n = Change rate");
   XDrawImageString(dpy,w,gc,20,65,str,strlen(str));
   strcpy(str,"c = Change caption");
   XDrawImageString(dpy,w,gc,20,80,str,strlen(str));
   strcpy(str,"a = About xpopclk");
   XDrawImageString(dpy,w,gc,20,95,str,strlen(str));
   strcpy(str,"<return> or <escape> = return/quit");
   XDrawImageString(dpy,w,gc,20,110,str,strlen(str));
   XFlush(dpy);
   /* get first date & time in seconds */
   time(&frstsec);
   currsec = frstsec;
   while (currsec == frstsec)
      {
      /* get first date & time in seconds */
      time(&currsec);
      slp();
      } /* loop until second starts */
   /* get first clock ticks since boot */
   prev = times(&t);
   frstsec = currsec;
   while (currsec == frstsec)
      {
      /* get current date & time in seconds */
      time(&currsec);
      slp();
      } /* loop for 1 second */
   /* get clock ticks since boot */
   clk = times(&t);
   if (clk < prev)
      {
      fprintf(stderr,"Negative ticks, rerun job\n");
      exit(0);
      }
   diff = clk - prev;
   printf("Ticks per second %d\n", diff);
   sleep(1);
   XSetFont(dpy,gc,fontid);
   XClearWindow(dpy,w);
   XSetBackground(dpy, gc, whiteColor);
   XSetForeground(dpy, gc, blackColor);
   return(diff);
   } /* gettps */

void prttime(void)
   {
   int i,hrs,mins,secs,len;
   double dblhour,dbljul;
   double greg2jd();
   char *asc;
   char str[64];
   time_t now,relsecs;
   struct tm *lcltm;
   XSetBackground(dpy, gc, whiteColor);
   XSetForeground(dpy, gc, blackColor);
   if (timesw == RELATIVE)
      {
      time(&now);
      relsecs = now - frstsecs;
      if (relsecs > totsecs)
         {
         totsecs = relsecs;
         hrs = relsecs / 3600;
         mins = (relsecs % 3600) / 60;
         secs = relsecs % 60;
         sprintf(str,"       %02d:%02d:%02d         ",
            hrs, mins, secs);
         XDrawImageString(dpy,w,gc,25,120,str,strlen(str));
         } /* if second elapsed */
      } /* relative time */
   else if (timesw == ASCII)
      {
      time(&now);
      relsecs = now - frstsecs;
      if (relsecs > totsecs)
         {
         totsecs = relsecs;
         lcltm = (struct tm *) localtime(&now);
         asc = (char *) asctime(lcltm);
         len = strlen(asc);
         asc[len-1] = '\0';
         XDrawImageString(dpy,w,gc,25,120,asc,len);
         } /* if second elapsed */
      } /* ascii */
   else if (timesw == JULIAN)
      {
      time(&now);
      relsecs = now - frstsecs;
      if (relsecs > totsecs)
         {
         totsecs = relsecs;
         lcltm = (struct tm *) localtime(&now);
         dblhour = (double) lcltm->tm_hour
            + (double) lcltm->tm_min / 60.0
            + (double) lcltm->tm_sec / 3600.0;
         dbljul = greg2jd(lcltm->tm_year+1900,
            lcltm->tm_mon+1,
            lcltm->tm_mday,
            dblhour);
         sprintf(str,"    %f         ", dbljul);
         XDrawImageString(dpy,w,gc,25,120,str,strlen(str));
         } /* if second elapsed */
      } /* julian */
   else
      { /* no time */
      if (timesw != prevtime)
         {
         prevtime = timesw;
         i = 0;
         while (i < 40) str[i++] = ' ';
         XDrawImageString(dpy,w,gc,25,120,str,strlen(str));
         } /* if change in time status */
      } /* no time */
   } /* prttime */


void count()
   {
   unsigned int diff;
   double tgtpop,currpop;
   time_t clk;
   struct tms t;
   char str[64];
   XSetBackground(dpy, gc, whiteColor);
   XSetForeground(dpy, gc, blackColor);
   currpop = totpop;
   if (fabs(totpop) < 1.0)
      {
      sprintf(str,"%10.0f", totpop);
      XDrawImageString(dpy,w,gc,50,80,str,strlen(str));
      } /* if total is zero */
   if (dbltkdlta < 0.0)
      tgtpop = totpop - 1.0;
   else
      tgtpop = totpop + 1.0;
   while (runflg && !returnsw
      && !resetsw && !ratesw)
      {
      if (dbltkdlta < 0.0
         && currpop < tgtpop) break;
      else if (dbltkdlta >= 0.0
         && currpop > tgtpop) break;
      clk = times(&t);
      if (clk < origclk)
         diff = clk + dayticks - origclk;
      else diff = clk - origclk;
      currpop = dbltkdlta * diff;
      slp();
      prttime();
      getkey();
      if (helpsw)
	 {
	 helpmenu();
	 break;
	 } /* if helpsw */
      if (aboutsw)
	 {
	 disclaim();
	 break;
	 } /* if aboutsw */
      if (titlesw)
	 {
	 chg_title();
	 break;
	 } /* if titlesw */
      } /* wait for birth */
   if (resetsw)
      {
      resetsw = 0;
      origclk = clk;
      currpop = 0.0;
      time(&frstsecs);
      totsecs = 0;
      } /* if reset */
   totpop = currpop;
   sprintf(str,"%10.0f", totpop);
   XDrawImageString(dpy,w,gc,50,80,str,strlen(str));
   prttime();
   } /* count */

void runclk()
   {
   while (runflg && !returnsw)
      {
      count();
      getkey();
      if (ratesw) chg_rate();
      if (titlesw) chg_title();
      if (helpsw) helpmenu();
      if (aboutsw) disclaim();
      if (runflg) slp();
      } /* display loop */
   } /* runclk */

int main(argc,argv)
int argc;
char **argv;
   {
   int tps;
   double dblspy,sign;
   double ten,unit;
   char annual_rate[128];
   char *p;
   struct tms t;
   if (argc < 2) putstx(*argv);
   if (strlen(*(argv+1)) > 20) putstx(*argv);
   strcpy(annual_rate,*(argv+1));
   p = annual_rate;
   if (*p == '-')
      {
      sign = -1.0;
      p++;
      }
   else sign = 1.0;
   dblanndlta = 0.0;
   ten = 10.0;
   while (*p != '\0')
      {
      if (isdigit(*p))
	 {
         unit = (double) *p - '0';
         dblanndlta = (dblanndlta * ten) + unit;
         p++;
	 } /* if numeric */
      else
	 {
         putstx(*argv);
	 } /* if not numeric */
      } /* for each digit in parm */
   if (dblanndlta >= 1.0e15)
      putstx(*argv);
   if (dblanndlta < 1.0e7)
      {
      fprintf(stderr,"Warning: low annual rate %.0f\n",
	 dblanndlta);
      fprintf(stderr,"Press <Return> to continue.\n");
      getchar();
      } /* if too small */
   strcpy(caption,"World Population Clock");
   cap_loc = 30;
   if (argc > 2)
      {
      int i,len;
      strcpy(caption,*(argv+2));
      len = strlen(caption);
      i = 3;
      while (i < argc &&
	 (len = len + 1 + strlen(*(argv+i))) < 127)
	 {
	 strcat(caption," ");
	 strcat(caption,*(argv+(i++)));
	 } /* for each word in title */
      cap_loc = (SCRW >> 1) - (((len >> 1) + 1) * 8);
      } /* if title on command line */
   /* Initialize X Windows */
   initx();
   /* calculate constants */
   /* annual change in population */
   dblanndlta *= sign;
   /* seconds per year */
   dblspy = 3600.0 * 24.0 * 365.25;
   /* ticks per second */
   tps = gettps();
   if (tps < 1 || tps > 999999)
      {
      fprintf(stderr,"Invalid tps %d\n", tps);
      exit(1);
      } /* if tps error */
   dbltps = (double) tps;
   /* ticks per year */
   dbltpy = dblspy * dbltps;
   /* population change per tick */
   dbltkdlta = dblanndlta / dbltpy;
   /* ticks per day */
   dayticks = 3600 * 24 * tps;
   XDrawImageString(dpy,w,gc,cap_loc,40,
      caption, strlen(caption));
   /* start counting now */
   timesw = prevtime = RELATIVE;
   origclk = times(&t);
   time(&frstsecs);
   resetsw = totsecs = helpsw =
      aboutsw = returnsw =
      titlesw = ratesw = 0;
   totpop = 0.0;
   runflg = 1;
   runclk();
   return(0);
   } /* main */
